#ifndef atomview_h
#define atomview_h

//Structure with the name of the element, its color, and the geode that it points to
//This is for ease of data handling
struct Colors
{
	//The name of the atom
	char* Name;
	//The geode that it points to 
	pfNode* ColorNode;
};
//Used to map atom name to atomcolor
struct ColorMap
{
	char* Name;
	char* Color;
};

//This is the real workhorse behind the atomview program
//You basically create one of these 
class AtomView
{
	public:
	//Init atomview
	AtomView(char** Env);
	//Destroy atomview
	~AtomView();
	//Start the performer display
	void StartDisplay();
	//Handle messages sent to the program
	string HandleMessages();
	private:
	//Actually start the performer data loop
	int RealDisplay();
	//Do some setup before doing the real gruntwork
	void InitDisplay();
	//Create lighting for the program
	void MakeLighting();
	//Map colors to each atom
	void MapColors();
	//Make the tree of atoms to display
	void MakeTree();
	//Update the tree of atoms
	void UpdateTree();
	//Get an array of the atom names
	void MapNames();
	//Are we supposed to go away?
	bool Die;
	//Are we stopped?
	bool Stop;
	//Are we repeating when playing?
	bool Loop;
	//Are we currently playing
	bool Playing;
	//Implements the teeth behind the messages
	void DoCommand(const string& ACommand);
	//The wrapper class for atoms, can be abstracted to hold any 
	//data though
	DTKSharedAtoms* AtomList;
	//The message parser
	CommandParser* message;
	//Send messages back to myself or more importantly , other processes running around waiting on info
	Messenger* SelfMessage;
	//Temporary atomview data objects used to hold the 
	//current set of atoms and for use to be written to  by the master data handler(AtomList)
	FlexArray<AtomDef> Curr;
	FlexArray<AtomDef> New;
	//Information about the atoms that are being displayed
	//What file are we at
	float Index; 
	//Beginning file
	float Begin; 
	//Ending file
	float Finish;
	//Play rate of the program (How many files a second to write to disk)
	double PlayRate;   //How many files per second to play
	//How many writes per second do we spit out?
	double DataRate;   
	//The number of polygons per sphere 
	int NumPolygons;   
	//The scale for each sphere
	double TheScale;   
	//The scale for the world
	double WorldScale; 
	//The size of the world before the change
	double OldWorldScale;
	//Has the world scale changed
	bool WorldScaleChange;
	//If the world is larger set this (for scaling)
	bool WorldLarger;
	//If the world is smaller set this (for scaling)
	bool WorldSmaller;
	//Has the scale for each sphere changed?
	bool ScaleChange;     
	//Are we scaling on the origin
	bool OriginScale;     
	//Are we scaling on the position
	bool PosScale;         
	//For catching singals in the display
	//The dpf object that holds everything, it is set in the RealDisplay function
	dpf *App; 
	//This is  used to tell if the atoms have changed
	bool AtomsChanged;
	//Used to tell what color goes where
	int NumColors;
	//This holds the color information for each atom type
	Colors* ColorData;
	//The origin cooridantes
	double x,y,z,h,p,r;
	//Has the origin changed?
	bool OriginChange;
	//GeoColor is the mapping of a color to a geode
	//This simplifies displaying objects immensley
	ColorHandler* GeoColor;
	//The environmental variables
	char** Environ;
	//The map between an atom and its color
	ColorMap* CMap;
	//The top node in the tree
	pfDCS* Scaler;
	//The array of pointers to atoms
	//This holds all of the atoms
	pfDCS** Display;
	//This is the number of atoms stored in the current tree
	int NumAtoms;
	//This is a listing of the atom names so I can keep track
	//of whether or not an atom actually changed
	char** AtomNames;
	//The number of names in the atom name  list
	int NumAtomNames;
	//The maximum number of atoms this class can handle
	int MaxAtoms;
};
//Legacy , left in just in case someone wants to make it work so that performers bad 
//signal handling can be replaced
void signal_catcher(const int& sig);
#endif
