#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <dpf.h>
#include <dtk.h>
#include <Performer/pf.h>
#include <Performer/pf/pfGeode.h>
#include <Performer/pr/pfMaterial.h>
#include <string>
using namespace std;
#include "FlexArrayTmpl.H"
#include "ReadAtomFiles.hh"
#include "SharedAtoms.h"
#include "CommandParser.h"
#include "Messenger.h"
#include "AtomCommands.h"
#include "colorhandler.h"
#include "atomview.h"

void AtomView::MapColors()
{
	//We only support 7 types now
	NumColors = 7;
	int NumTypes = AtomList->FileData->atom_types[(int)AtomList->Curr()].nTypes();
	if (NumTypes > 7)
		NumTypes = 7;
	NumColors = NumTypes;
	//Go through and get the names
	//Set the names both in the color data and in the color map
	for (int i=0; i< NumTypes; i++)
		strcpy(ColorData[i].Name ,AtomList->FileData->atom_types[(int)AtomList->Curr()].getTypeName(i));
	for (int i=NumTypes; i<7; i++)
		strcpy(ColorData[i].Name, "--");
//	for (int i=0; i<NumTypes; i++)
//		printf("%s has color %s\n", ColorData[i].Name, CMap[i].Color);
	//Associate the right geode with the right color
	for (int i=0; i<NumColors; i++)
	{
		for (int j=0; j<NumColors; j++)
		{
			if (!strcmp(ColorData[i].Name, CMap[j].Name))
				ColorData[i].ColorNode = GeoColor->GetColor(CMap[j].Color);
		}
	}
//	for (int i=0; i<7; i++)
//		printf("%s %s %s\n", CMap[i].Name, CMap[i].Color, ColorData[i].Name);
//	printf("I have %d types\n", NumTypes);

}


void AtomView::MakeTree()
{
	//Get the correct number of atoms to display
	NumAtoms = New.getSize();
	//Setup the array to be used to keep track of atom names
	MaxAtoms = NumAtoms+1000;
	AtomNames = new char*[MaxAtoms];
	for (int i=0; i< MaxAtoms; i++)
	{
		AtomNames[i] = new char[2];
		strcpy(AtomNames[i], "--");
	}
	NumAtomNames = NumAtoms;
	//Correctly map atoms to their respective colors
	for (int i=0; i<7; i++)
	{
		if (ColorData[i].Name != NULL)
			strcpy(ColorData[i].Name, "--");
	}
	//printf("Make the geodes\n");
	if (GeoColor == NULL)
		GeoColor = new ColorHandler(NumPolygons);
	//Stuff the default colors into the color map 
	for (int i=0; i<7; i++)
		ColorData[i].ColorNode = GeoColor->GetColor(CMap[i].Color);
	MapColors();
	//printf("After mapping colors\n");
	MapNames();
	//Create the scale node

	//Make a bunch of node pointers to hold the atoms
	//Display = new pfDCS*[MaxAtoms];
	Display = (pfDCS**)malloc(sizeof(pfDCS*)*MaxAtoms);
	for (int i=0; i< MaxAtoms ; i++)
	{
		Display[i] = new pfDCS();
		//Display[i] = (pfDCS*)malloc(sizeof(pfDCS));
	}
	//Set the color of the each atom
	//Colordata has at most 7 entries so I basically just
	//access which color to put in by going to what type it
	//is and getting the color from that
	for (int i=0; i<NumAtoms; i++)
		Display[i]->addChild(ColorData[New[i].type].ColorNode);

	//This is the node underneath the world that allows me to scale the world easily
	if (Scaler !=NULL)
		delete Scaler;

	Scaler = new pfDCS;
	//Set the scale of the node to be what we want
	Scaler->setScale(TheScale);

	//Go through and set up the atoms at their correct x,y,z points and add them below the scale node
	for (int i=0; i< NumAtoms; i++)
	{
		Display[i]->setTrans(New[i].p.cart.x, New[i].p.cart.y, New[i].p.cart.z);
		Display[i]->setScale(TheScale);
		Scaler->addChild(Display[i]);

	}
	//Add all the other nodes to the tree but dont' show anything
	for (int i=NumAtoms; i<MaxAtoms; i++)
		Scaler->addChild(Display[i]);
}

void AtomView::UpdateTree()
{
	bool FlushColor = false;
	if (New.getSize() != NumAtoms)
		FlushColor = true;
	if (New.getSize() > MaxAtoms)
	{
		//printf("Doing the realloc old max size is %d and the new size is %d!\n", MaxAtoms, New.getSize());
		
		Display = (pfDCS**)realloc(Display, sizeof(pfDCS*)*New.getSize());
		for (int i=MaxAtoms; i< New.getSize(); i++)
		{
			//Display[i] = (pfDCS**)malloc(sizeof(pfDCS));
			Display[i] = new pfDCS;
			//usleep(1000);
			if (Display[i] == NULL)
				printf("WHY AM I NULL?\n");
			Scaler->addChild(Display[i]);

		}
		//printf("Display alloc'd!\n");
		char** TempNames = new char*[New.getSize()];

		for (int i=0; i<MaxAtoms; i++)
			TempNames[i] = AtomNames[i];
		//printf("Atoms copied ok\n");
		for (int i=MaxAtoms; i< New.getSize(); i++)
		{
			TempNames[i] = new char[2];
			strcpy(TempNames[i], "--");
		}
		//delete [] AtomNames;
	//	printf("New atoms added ok\n");
		AtomNames = TempNames;
		MaxAtoms = New.getSize();
	}
	if (New.getSize() > MaxAtoms)
	{
		printf("*****************ERROR*******************\n");
		printf("***********SOMETHING WENT WRONG**********\n");
		printf("***DO NOT TRUST PROGRAM RESULTS ANYMORE**\n");
		printf("*****************************************\n");
	}
	//Map the colors to the correct atoms
	MapColors();
	//printf("Map color ok\n");
	//Clear out old atoms if needbe
	int OldSize = NumAtoms;
	//Remove geodes from the display list so they don't show up
	if (New.getSize() < NumAtoms)
	{
	//	printf("Removing atoms\n");
		while (NumAtoms > New.getSize())
		{
			if (Display[NumAtoms-1] !=NULL)
			{
			//Scaler->removeChild(Display[NumAtoms-1]);
			Display[NumAtoms-1]->removeChild(Display[NumAtoms-1]->getChild(0));
			}
			strcpy(AtomNames[NumAtoms-1] , "--");
			NumAtoms--;
		}
	}
//	printf("After removing Old # = %d New # = %d\n", OldSize, NumAtoms);
	OldSize = NumAtoms;
	int NumBad =0;
	//Add new nodes to the tree if needbe
	if (New.getSize() > NumAtoms)
	{
		while (NumAtoms < New.getSize() )
		{
	//		printf("NumAtoms = %d && MaxAtoms = %d\n", NumAtoms, MaxAtoms);
			if (Display[NumAtoms] != NULL)
			{
			//Add the right display item to the nod
			Display[NumAtoms]->addChild(ColorData[New[NumAtoms].type].ColorNode);
			//Set its position
			Display[NumAtoms]->setTrans(New[NumAtoms].p.cart.x, New[NumAtoms].p.cart.y, New[NumAtoms].p.cart.z);
			//Set its scale
			Display[NumAtoms]->setScale(TheScale);
			//Add the name to the atom name list
			strcpy(AtomNames[NumAtoms], ColorData[New[NumAtoms].type].Name);
			//Increase the number of atoms
			}
			else
			{
				//printf("Bad atom at %d\n", NumAtoms);
				NumBad++;
			}
			//	printf("Null atom at %d\n", NumAtoms);
			NumAtoms++;
		}
	}
	//printf("I HAVE %d BAD ATOMS\n", NumBad);
	//printf("After adding Old # = %d New # = %d\n", OldSize, NumAtoms);
	int NumReplace=0;
	NumAtoms = New.getSize();
//	printf("Number of types = %d\n",AtomList->FileData->atom_types[(int)AtomList->Curr()].nTypes());
	//printf("Atom position = %lf\n", AtomList->Curr());
	for (int i=0; i<NumAtoms; i++)
	{
		if (Display[i] ==NULL)
			continue;
		//If the atom has a different name then delete it
		//and put in the correct one
		if (strcmp(AtomNames[i], ColorData[New[i].type].Name) !=0 || FlushColor)
		{
			NumReplace++;
			//Add the right display item to the nod
			Display[i]->replaceChild(Display[i]->getChild(0), ColorData[New[i].type].ColorNode);
			//Set its position
			Display[i]->setTrans(New[i].p.cart.x, New[i].p.cart.y, New[i].p.cart.z);
			//Set its scale
			Display[i]->setScale(TheScale);
			//Add the name to the atom name list
			strcpy(AtomNames[i], ColorData[New[i].type].Name);
		}
	}
	//Set the names of the atoms for the next update
	MapNames();
	//printf("Replaced %d atoms\n", NumReplace);
	//printf("I now have %d atoms in the world\n", NumAtoms);
	//Update the xyz positions
	for (int i=0; i< NumAtoms; i++)
	{
		if (Display[i] != NULL)
			Display[i]->setTrans(New[i].p.cart.x, New[i].p.cart.y, New[i].p.cart.z);
	}

	//Update the names of the atoms
}

void AtomView::MapNames()
{
	if (NumAtoms > MaxAtoms)
	{
		printf("*****************ERROR*******************\n");
		printf("***MORE THAN 1000 ATOMS HAVE BEEN ADDED**\n");
		printf("***DO NOT TRUST PROGRAM RESULTS ANYMORE**\n");
		printf("*****************************************\n");
	}
	int NumTypes = AtomList->FileData->atom_types[(int)AtomList->Curr()].nTypes();
	for (int i=0; i<NumAtoms; i++)
	{
		/*
		printf("Type is= %d\n",New[i].type);

		printf("Name1 = %s\n",AtomNames[i]);
		printf("Name2 = %s\n",ColorData[New[i].type].Name);
		printf("Numatoms = %d and Real size is %d\n", NumAtoms, New.getSize());
		printf("At position = %d %d \n", i,  NumTypes);
		*/
		strcpy(AtomNames[i], ColorData[New[i].type].Name);
	}
	NumAtomNames = NumAtoms;
}
