#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <dpf.h>
#include <dtk.h>
#include <Performer/pf.h>
#include <Performer/pf/pfGeode.h>
#include <Performer/pr/pfMaterial.h>
#include <string>
using namespace std;
#include "FlexArrayTmpl.H"
#include "ReadAtomFiles.hh"
#include "SharedAtoms.h"
#include "CommandParser.h"
#include "Messenger.h"
#include "AtomCommands.h"
#include "colorhandler.h"
#include "atomview.h"


void AtomView::StartDisplay()
{
	//signal(SIGINT, signal_catcher);
	//signal(SIGTERM, signal_catcher);
	pfSharedArenaSize((1024*1024*768));
	if(pfInit() != 1)
	{
		dtkMsg.add(DTKMSG_ERROR, "pfInit() failed.\n");
		return; // error
	}
	RealDisplay();
	if (!pfIsConfiged()) pfConfig();
	usleep(200000);
	dtkMsg.add(DTKMSG_INFO, "calling pfExit().\n");
	pfExit();
	dtkMsg.add(DTKMSG_INFO, "called pfExit().\n");
}

void AtomView::MakeLighting()
{
  /* Begin Geode creation */
   pfSCS *lightSCS;
   pfMatrix mat;
   pfGeoState *gstate = new pfGeoState;
   pfLightSource* ls;
   gstate->setMode(PFSTATE_ENLIGHTING, PF_ON);
   gstate->setMode(PFSTATE_CULLFACE, PFCF_OFF);
   App->display()->scene()->setGState(gstate);
   mat.makeRot(45,1,0,0);
   App->display()->scene()->addChild(lightSCS = new pfSCS(mat));
   lightSCS->addChild(ls = new pfLightSource);
   ls->on();
}


int AtomView::RealDisplay()
{

  // You may want to replace DSO keyboardInput with keyboardMouseInput
  // so you have X mouse ("buttons" and "pointer") input and X
  // "keyboard" input.  Run: (cd `dpf-config --dso-dir`;ls *.so) and
  // (cd `dtk-config --augment-dso-dir`;ls *.so) to see a list of all
  // installed dpf and dtk DSO's. Loading a dpf Display DSO will call
  // dpfDisplay::dpfDisplay() which will in turn call pfInit() if
  // pfInit() has not been called already.
  dpf app("desktopCaveSimGroup", NULL);

  // Check for error. If not for main() calling Main(), pfExit() would
  // not get called.
  if(app.isInvalid()) return 1;

  // So signal_catcher() can get to dpf::state.
  App = &app;

  // If you load other kinds of files you'll need to call
  // pfdInitConverter(".suffix").
  if(!pfdInitConverter(".pfb"))
    {
      dtkMsg.add(DTKMSG_ERROR,
		 "pfdInitConverter(\".pfb\") failed.\n");
      return 1;
    }

  // This will call all the loaded DSO dtkAugments preConfig() methods.
  if(app.preConfig()) return 1; // error

  // app.config() calls pfConfig() if you have a Performer display
  // loaded, which create pipes, and may forks into separate processes
  // most Performer shared data must be set up before this call.
  // app.config() calls app.preConfig() for you if you have not
  // already.  We called app.preConfig() in order to catch possible
  // errors sooner.
  if(app.config()) return 1;

  // Replace Performer's bugy process hanging SIGCHLD catcher.
  //signal(SIGCHLD, signal_catcher);

  // app.postConfig() will call all the loaded DSO dtkAugments
  // postConfig() methods.  app.postConfig() will call app.preConfig()
  // and app.config(), if they have not been called yet.
  if(app.postConfig()) return 1; // error

   pfFilePath(".:"DPF_DATA_DIR);
  //Johns post config
  //Open shared memory (what I am writing to with the atom view
  //Figure out number of atoms
  //Number of atom types
  //dtkSharedMem DataSource("ATOM_DATA");
  //int NumAtoms = DataSource.getSize()/(sizeof(float)*3+sizeof(int)*1);

//Setup the lighting in the model
MakeLighting();

   //Make a tree of nodes to display under the world
   MakeTree();
   //Add the tree of nodes to the world
   app.display()->world()->addChild(Scaler);
   //While I am supposed to display do this
  while(app.state & DTK_ISRUNNING)
  {
      if(app.frame()) return 1; // error
  	//Do message upkeep so everything runs smoothly
      	HandleMessages();
	//If we are changing the size of the atoms do it here
	if (ScaleChange)
	{
		printf("Setting the atoms to be of scale %lf\n", TheScale);
	        for (int i=0; i<NumAtoms; i++)
			Display[i]->setScale(TheScale);
		ScaleChange = false;
	}
	//If we are changing the scale of the world do this
	if (WorldScaleChange)
	{
		printf("Scaling the world to %f\n", WorldScale);
		//If we are scaling based on the current position, set it as the origin then scale
		//The origin is preserved in member variables 
		if (PosScale)
		{
			//Get current xyz,hpr and then set it as the origin
			/*
			if(app.display()->currentNav)
			{
				printf("Scaling based on current position!\n");
				dpfNav *foo = app.display()->currentNav ;
				if (foo) 
				{
					printf("Scale b4 = %lf\n", foo->scale);
					//foo->scale = scaleAroundMeValue ;
					foo->scale = WorldScale;
					printf("Scale after =%lf\n", foo->scale);
				}

			}
			*/
		}
		else
		{
			printf("Scaling on orgin\n");
			Scaler->setScale(WorldScale);
		}
		WorldScaleChange = false;
	}
	//If our origin has changed, set the new origin
	if (OriginChange)
	{
		printf("I am changing the position of the orgin to %lf %lf %lf %lf %lf %lf\n", x,y,z,h,p,r);
		/*
		app.display()->currentNav->location.xyz[0] = x;
		app.display()->currentNav->location.xyz[1] = y;
		app.display()->currentNav->location.xyz[2] = z;
		app.display()->currentNav->location.hpr[0] = h;
		app.display()->currentNav->location.hpr[1] = p;
		app.display()->currentNav->location.hpr[2] = r;
		*/
		OriginChange = false;
	}

	//Check to see if any of the atoms changed, if they did set the transition
	//If the atoms changed update our current copy with the new copy
	if (AtomsChanged)
	{
		printf("ATOMS CHANGED\n");
		UpdateTree();
		AtomsChanged = false;
	}
  }
  return 0; // success
}

//Catches signals!
void signal_catcher(int sig)
{
  dtkMsg.add(DTKMSG_INFO,
	     "PID %d caught signal %d, starting exit sequence ...\n",
	     getpid(), sig);
#ifdef Linux
  /* All this crap is to get around Performer bugs.  The Performer 2.5
   * calls exit from pfFrame() sometimes, and do to a race conditions
   * on multi-processor Linux boxes, will not let your program exit
   * properly.  So to prevent this we add this mess.  John please
   * don't remove this without letting me test your fix, on 2 CPU
   * Linux box, first. */
	int parent_pid = 0;
  if(getpid() != parent_pid)
    {
      signal(SIGINT, NULL);
      signal(SIGTERM, NULL);
      dtkMsg.add(DTKMSG_DEBUG, "%d forking.\n", getpid());
      // Make an evil child that slowly kills its parent.
      pid_t pid = fork();
      if(pid == (pid_t) 0) // child
	{
	  pid = getppid();
	  kill(pid, SIGCONT);
	  usleep(200000); // 2/10 seconds sleep
	  dtkMsg.add(DTKMSG_DEBUG,
		     "child pid %d killing %d\n",
		     getpid(), pid);
	  kill(pid, sig); // kill your parent.
	  exit(0);
	}
      if(pid < (pid_t) -1)
	dtkMsg.add(DTKMSG_WARN, 1, "fork() failed.\n");
      return; // parent
    }
  else if(sig == SIGCHLD)
    {
      // If the Performer code catches SIGCHLD it will call exit() and
      // not let your program clean-up.
      int status;
      wait(&status);
    }

#ifdef IRIX
/* IRIX removes the signal_catcher after the signal.  We put it back
 * here . */
  signal(sig, signal_catcher);
#endif
#endif
  // unset the dpf::state DTK_ISRUNNING state bit to cause the APP
  // process in the main loop to exit.
//  App->state &= ~(DTK_ISRUNNING);
}
