#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <dpf.h>
#include <dtk.h>
#include <Performer/pf.h>
#include <Performer/pf/pfGeode.h>
#include <string>
#include <sstream>
using namespace std;
#include "AtomCommands.h"
#include "CommandParser.h"
#include "Messenger.h"
#include "FlexArrayTmpl.H"
#include "ReadAtomFiles.hh"
#include "SharedAtoms.h"
#include "colorhandler.h"
#include "atomview.h"

//Top level handler function that does all of the message handling and deletion
string AtomView::HandleMessages()
{
	string ACommand; //Temporary command holder
	//If we have messagers that are not any good then create them and hook them up
	//to ATOM_MESSAGE.  ATOM_MESSAGE is the shared memory filename that I (Andrew A. Ray)
	//have used for all of the message passing in this program
	if (message ==NULL)
		message = new CommandParser("ATOM_MESSAGE");
	if (SelfMessage == NULL)
		SelfMessage = new Messenger("ATOM_MESSAGE");

	//Get the message out of the message class by Mark J. Soltys II  (Modified by Andrew A. Ray to get around
	//   the use of environmental variables)
	message->getCommand(ACommand, CLASS_TYPE); 
	//Call a subfunction to handle all of the actual message implementation
	DoCommand(ACommand);
	//If we are supposed to go away do so and clean up after ourselves
	if (Die)
	{
		printf("Finished doing commands exiting\n");
		delete message;
		delete SelfMessage;
	}
	//For ease of use
	return ACommand;
}

//This actually implements all of the commands this program accepts
//It is rather large and lengthy, but is necessary 
void AtomView::DoCommand(const string& ACommand)
{
	//Debug
	if (ACommand != "")
		printf("Doing command %s\n", ACommand.c_str());
		//Exit the program
		if (ACommand == DIE)
		{
			Die=true;
			return;
		}
		//Scale the atom size
		if (ACommand == MODELSCALE)
		{
			ScaleChange = true;
			TheScale = message->getDouble();
			return;
		}
		//Scale the world size
		if (ACommand == WORLDSCALE)
		{
			OldWorldScale = WorldScale;
			WorldScale = message->getDouble();
			if (OldWorldScale == WorldScale)
				return;
			if (OldWorldScale > WorldScale)
			{
				WorldLarger = false;
				WorldSmaller = true;
			}
			else
			{
				WorldSmaller=false;
				WorldLarger=true;
			}
			WorldScaleChange = true;
			return;
		}
		//Set the number of polygons
		if (ACommand == NUMPOLY)
		{
			NumPolygons = message->getInt();
			return;
		}
		//Do nothing, can be used for sleeping ,initialization, etc...
		if (ACommand == NOTHING)
			return;
		//Goto dataset+ 1
		if (ACommand == READNEXT)
		{
			printf("Going to the next file\n");
			AtomList->GoForward();
			AtomList->WriteToSharedMem();
			return;
		}
		//Goto dataset -1 
		if (ACommand == READPREVIOUS)
		{
			printf("Going to the previous file\n");
			AtomList->GoBackward();
			AtomList->WriteToSharedMem();
			return;
		}
		//Read the data set at a specific time
		if (ACommand == READTIME)
		{
			AtomList->Goto((float)message->getDouble());
			printf("Got to read time, reading at %f\n", AtomList->Curr());
			AtomList->WriteToSharedMem();
			return;
		}
		//Refresh the current data (mainly for paranoia)
		if (ACommand == READCURR)
		{
			AtomList->WriteToSharedMem();
			return;
		}
		//Put the first data set into memory
		if (ACommand == READFIRST)
		{
			printf("Got the goto first command, going to %f\n", Begin);
			AtomList->Goto(Begin);
			AtomList->WriteToSharedMem();
			return;
		}
		//Read the last data set
		if (ACommand == READLAST)
		{
			AtomList->Goto(Finish);
			AtomList->WriteToSharedMem();
			return;
		}
		//Set the play rate 
		if (ACommand == PLAYRATE)
		{
			PlayRate = message->getDouble();
			return;
		}
		//Set the information for playing
		if (ACommand == PLAY) 
		{
			printf("The display program got the play command\n");
			Stop = false;
			Playing = true;
			return;
		}
		//Package up information so outside programs can see
		//what information I have stored
		//This sends out (in this order)
		//PlayRate, DataRate, Current position in the dataset, Ending position in the dataset, Are we looping, Are we playing, Are we stopped,
		//Are we scaling on our Origin, What is the current scale value for the world, What is the current scale of the atoms
		if (ACommand == GETINFO)
		{
			printf("Beginning the getinfo command\n");
			char* Buff = new char[400];
			sprintf(Buff, "%s\t%f\t%f\t%f\t%f\t%d\t%d\t%d\t%d\t%d\t%lf\t%lf",INFOIS.c_str(), PlayRate, DataRate, AtomList->Curr(), Finish, Loop, Playing, Stop,OriginScale,PosScale, WorldScale, TheScale);
//			output << INFOIS << "\t" << PlayRate << "\t" << DataRate << "\t" << AtomList->Curr();
			printf("Doing GETINFO, data in thingy is %s\n", Buff);
			SelfMessage->SendCommand(string(Buff));
			delete [] Buff;
			return;
		}

		//Set the data rate
		if (ACommand == DATARATE)
		{
			DataRate = message->getDouble();
			return;
		}
		//Stop the playing
		if (ACommand == STOP)
		{
			Loop = false;
			Playing = false;
			Stop = true;
			return;
		}
		//This sets the loop variable (if its off it turns it on, if it is on it turns it off)
		if (ACommand == LOOP )
		{
			if (Loop)
				Loop = false;
			else
				Loop = true;
			return;
		}
		//Stop the navigation
		if (ACommand == STOPNAV)
		{
			dpfPipe *p ;
			dpfPwin *w ;
			pfPipeWindow *pw ;
			for (p=App->display()->first(); (p); p=p->next())
			{
				for (w=p->first(); (w); w=w->next())
				{
					pw = w->pfpwin() ;
					pfuLoadPWinCursor(pw,PFU_CURSOR_X) ;
				}
			}
			//App->display()->currentNav->deactivate();
			return;
		}
		//Start the navigation
		if (ACommand == STARTNAV)
		{
			printf("Starting navigation, add the code in to make this happen\n");
			dpfPipe *p ;
			dpfPwin *w ;
			pfPipeWindow *pw ;
			for (p=App->display()->first(); (p); p=p->next())
			{
				for (w=p->first(); (w); w=w->next())
				{
					pw = w->pfpwin() ;
					pfuLoadPWinCursor(pw,PFU_CURSOR_OFF) ;
				}
			}
			//App->display()->currentNav->activate();
			return;
		}
		//Get the current file position we are at in the dataset
		if (ACommand == GETTIME)
		{
			string Temp = TIMEIS;
			char* Buff = new char[100];
			sprintf(Buff, "%lf", AtomList->Curr());
			Temp = Temp + "\t" + string(Buff);
			SelfMessage->SendCommand(Temp);
			delete [] Buff;
			return;
		}
		//Set the number of polygons to display with
		if (ACommand == NUMPOLYGONS)
		{
			NumPolygons = message->getInt();
			printf("Setting number of polygons to %d\n", NumPolygons);
			return;
		}
		//Goto the original position
		if (ACommand == GOTOORIGIN)
		{
			printf("Going to orgin, implement me!\n");
			//printf("Pivioting on %d %d %d %d %d %d\n", x,y,z,h,p,r);
			OriginChange = true;
			return;
		}
		//Scale based on the current position, not the origin
		if (ACommand == SCALEPOS)
		{
			PosScale = true;
			OriginScale = false;
			printf("Scaling on current position\n");
			return;
		}
		//Scale based on the origin
		if (ACommand == SCALEORIGIN)
		{
			PosScale = false;
			OriginScale = true;
			printf("Scaling on the original\n");
			return;
		}
		//Set a specific atoms color
		if (ACommand == ATOMCOLOR)
		{
			//Get the data
			string AtomName  = message->getString();
			string AtomColor = message->getString();
			//Go through the atom list and see if the atom actually exists
			//if it does then set its color to be equal to what was passed in
			bool Found = false;
			for (int i=0; i< NumColors; i++)
			{
				if (!strcmp(CMap[i].Name, AtomName.c_str()))
				{
					Found = true;
					if (!strcmp(CMap[i].Color, "--"))
						strcpy(CMap[i].Color , AtomColor.c_str());
				}
			}
			printf("Did not find color\n");
			if (!Found)
			{
				for (int i=0; i<7; i++)
				{
					if (!strcmp(CMap[i].Name, "--"))
					{
						strcpy(CMap[i].Color , AtomColor.c_str());
						strcpy(CMap[i].Name  , AtomName.c_str());
						break;
					}
				}
			}
			return;
		}
		//Setup the file information for the atom data and then determine what color goes to what
		//atom
		if (ACommand == FILEINFO)
		{
			printf("Got the file info command\n");
			string BaseName = message->getString();
			int Start = message->getInt();
			int End = message->getInt();
			AtomList = new DTKSharedAtoms("ATOM_MESSAGE", (char*)BaseName.c_str(), Start, End, New, AtomsChanged);
			printf("I was created ok?\n");
			Begin =0; 
			Finish = AtomList->End();
			printf("File information processed\n");
		}
		//Set the orgin of the 
		if (ACommand == SETORIGIN)
		{
			//int x=0,y=0,z=0,h=0,p=0,r=0;
			x=message->getDouble();
			y=message->getDouble();
			z=message->getDouble();
			h=message->getDouble();
			p=message->getDouble();
			r=message->getDouble();
			printf("Got set origin to %lf %lf %lf %lf %lf %lf\n", x,y,z,h,p,r);
			OriginChange = true;
			return;
		}
}
