#include <Performer/pfdu.h>
#include <Performer/pf/pfGeode.h>
#include <Performer/pr/pfGeoSet.h>
#include <Performer/pr/pfMaterial.h>

//Figure out which ones I do not need later :P
#include <string.h>
#include "colorhandler.h"

//Create a class instance w/ 7 colors in it!
ColorHandler::ColorHandler(const int& NumPolygons)
{
	//This is fixed so we create the 7 necessary colors for use in atomview :)
	NumColors = 7;
	//Create the 7 geodes
	TheGeodes = new pfGeode*[NumColors];
	//Allocate memory for them
	for (int i=0; i<NumColors; i++)
		TheGeodes[i] = new pfGeode();

	//Make the geo state 
	TheGState = new pfGeoState;
	//Make a pfMaterial object and set its attributes
	TheMaterial = new pfMaterial;
	TheMaterial->setShininess(100.0);
	TheMaterial->setColorMode(PFMTL_BOTH,PFMTL_CMODE_AMBIENT_AND_DIFFUSE);
	//Set the geo states attributes to include the material
	TheGState->setAttr(PFSTATE_FRONTMTL, TheMaterial);

	//Create the geo sets
	TheGeoSets = new pfGeoSet*[NumColors];
	//Allocate memory for them
	for (int i=0; i<NumColors; i++)
		TheGeoSets[i] = pfdNewSphere(NumPolygons, pfGetSharedArena());

	//Create the colors
	TheColors = new pfVec4*[NumColors];
	//Set the color information
	for (int i=0; i<NumColors; i++)
		TheColors[i] = new pfVec4;

	//Create the names for all of the colors
	TheNames= new char*[NumColors];
	//Allocate memory for the names (if you use more than 50 chars to describe a color you deserve to be shot)
	for (int i=0; i<NumColors; i++)
		TheNames[i] = new char[50];
    
	//Associate names with the colors
	strcpy(TheNames[0], "blue");
	strcpy(TheNames[1], "red");
	strcpy(TheNames[2], "green");
	strcpy(TheNames[3], "yellow");
	strcpy(TheNames[4], "orange");
	strcpy(TheNames[5], "white");
	strcpy(TheNames[6], "purple");
	//Set their rgb values
	//Blue
	TheColors[0]->set(0.2, 0, .8, 1); 
	//Red
	TheColors[1]->set(1,0,0,1);
	//Green
	TheColors[2]->set(0, 1, 0, 1);
	//Yellow
	TheColors[3]->set(1, 1, 0, 1); 
	//Orange
	TheColors[4]->set(1, .5, 0, 1); 
	//White
	TheColors[5]->set( 1,1,1,1);
	//Purple
	TheColors[6]->set(0.6275 ,  0.1255  , 0.9412,1);

	//Add the colors to the geosets
	for (int i=0; i<NumColors; i++)
		TheGeoSets[i]->setAttr(PFGS_COLOR4, PFGS_OVERALL, TheColors[i], NULL);
	//Add the lighting to the geoset
	for (int i=0; i<NumColors; i++)
		TheGeoSets[i]->setGState(TheGState);
	//Add the geoset to the geode
	for (int i=0; i<NumColors; i++)
		TheGeodes[i]->addGSet(TheGeoSets[i]);
}
//Clean up all of the memory this thing allocates
ColorHandler::~ColorHandler()
{
	for (int i=0; i<NumColors; i++)
	{
		delete TheGeodes[i];
		delete TheGeoSets[i];
		delete TheNames[i];
		delete TheColors[i];
	}
	delete TheGeodes;
	delete TheGeoSets;
	delete TheNames;
	delete TheGState;
	delete TheMaterial;
	delete TheColors;
}

//Get a color out of this thing
pfGeode* ColorHandler::GetColor(char* Color)
{
	//Safety check
	if (Color == NULL)
		return NULL;
	//Loop through the colors and return the proper geode if everything works
	for (int i=0; i<NumColors; i++)
	{
		if (!strcmp(Color, TheNames[i]))
			return TheGeodes[i];
	}
	return NULL;
}

void ColorHandler::AddColor(const char* const Name, const double& r, const double& b, const double& g)
{
	printf("Will be implemented later if needed\n");
	return;
}
